"""

 MIT License

 Copyright (c) 2020 Benjamin Collins (kion @ dashgl.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.

"""

class NinjaBone: # {

	def __init__(self, flag):
		self.flag = flag
		self.localMatrix = self.identity() 
		self.worldMatrix = self.identity()
		return None

	def indentity(self):
		return [
			[ 1, 0, 0, 0 ],
			[ 0, 1, 0, 0 ],
			[ 0, 0, 1, 0 ],
			[ 0, 0, 0, 1 ]
		]

	def multiply(self, a, b):
		tmp = self.identity()
		for i in range(4):
			for j in range(4):
				t = 0.0
				for k in range(4):
					t = t + (a[i][k] * b[k][j])
				tmp[i][j] = t
		return tmp

	def multiplyLocal(self, factor):
		a = self.worldMatrix
		b = factor
		tmp = self.multiply(a, b)
		self.worldMatrix = tmp
		return None

	def multiplyWorld(self, factor):
		a = self.localMatrix
		b = factor
		tmp = self.multiply(a, b)
		self.localMatrix = tmp
		return None

	def save(self):
		for i in range(4):
			for k in range(4):
				self.worldMatrix[i][k] = self.localMatrix[i][k]
		return None

	def setScale(self, vec3):
		tmp = self.identity()
		tmp[0][0] = vec3[0]
		tmp[1][1] = vec3[1]
		tmp[2][2] = vec3[2]
		self.multiplyLocal(tmp)
		return None

	def setRotation(self, vec3, zxy_order):
		if zxy_order:
			z = vec3[0]
			x = vec3[1]
			y = vec3[2]
		else:
			x = vec3[0]
			y = vec3[1]
			z = vec3[2]

		# rotate x-axis
		tmp = self.identity()
		c = math.cos(x)
		s = math.sin(x)
		tmp[1][1] = c;
		tmp[1][2] = s;
		tmp[2][1] = -s;
		tmp[2][2] = c;
		self.multiplyLocal(tmp)

		# rotate y-axis
		tmp = self.identity()
		c = math.cos(y)
		s = math.sin(y)
		tmp[0][0] = c;
		tmp[0][2] = -s;
		tmp[2][0] = s;
		tmp[2][2] = c;
		self.multiplyLocal(tmp)

		# rotate z-axis
		tmp = self.identity()
		c = math.cos(z)
		s = math.sin(z)
		tmp[0][0] = c;
		tmp[0][1] = s;
		tmp[1][0] = -s;
		tmp[1][1] = c;
		self.multiplyLocal(tmp)
		return None

	def setPosition(self, vec3):
		tmp = self.identity()
		tmp[0][0] = vec3[0]
		tmp[1][1] = vec3[1]
		tmp[2][2] = vec3[2]
		self.multiplyLocal(tmp)
		return None

