"""

 MIT License

 Copyright (c) 2020 Benjamin Collins (kion @ dashgl.com)

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in all
 copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.

"""

import os
import struct
from NinjaBone import NinjaBone

class NinjaModel: # {

	def __init__(self, filename): # {

		# File Information
		self.path = 'input/' + filename
		self.file = open(self.path, 'rb')
		self.length = os.path.getsize(self.path)

		# Textures
		self.tex_list = []

		# Bone List
		self.bones = []

		return None
	# }

	def parse(self): # {
		
		while self.file.tell() < self.length: # {
			
			bytes = self.file.read(4)

			if(bytes == 'NJTL') :
				print('Found Ninja Texture List')
				bytes = self.file.read(4)
				len = struct.unpack('I', bytes)[0]
				pos = self.file.tell() + len
				self.readNjtl()
				self.file.seek(pos, 0)
			elif (bytes == 'NJCM') :
				print('Found Ninja Chunk Model')
				bytes = self.file.read(4)
				len = struct.unpack('I', bytes)[0]
				pos = self.file.tell() + len
				self.pof = self.file.tell()
				self.readNjcm()
				self.file.seek(pos, 0)
			elif (bytes == 'NMDM') :
				print('Found Ninja Direct Motion')

		# }
		
		return None
	# }

	def readNjtl(self): # {
		
		pof = self.file.tell()

		bytes = self.file.read(8)
		n = struct.unpack('II', bytes)
		
		str_ofs = []

		self.file.seek(n[0] + pof, 0)
		for i in range(0, n[1]): # {
			bytes = self.file.read(12)
			m = struct.unpack('III', bytes)
			str_ofs.append(m[0] + pof)
		# }

		for ofs in str_ofs: # {
			
			str = ''
			self.file.seek(ofs, 0)

			while 1: # {
				
				ch = self.file.read(1)
				if(ch == '\0'):
					break
				str += ch
			#} 
			
			self.tex_list.append(str)

		# }
		
		return None
	# }

	def readNjcm(self, parentBone): # {
		
		bytes = self.file.read(52)
		b = struct.unpack('IIfffiiifffII', bytes)

		flags = b[0]
		model_ofs = b[1]

		pos = ( b[2], b[3], b[4] )
		rot = ( b[5], b[6], b[7] )
		scl = ( b[8], b[9], b[10] )

		child_ofs = b[11]
		sibling_ofs = b[12]
		
		self.bone = NinjaBone()

		if ( (flags & 0x04) == 0):
			self.bone.setScale(scl)

		if ( (flags & 0x02) == 0):
			self.bone.setRotation(rot, flags & 0x20)

		if ( (flags & 0x01) == 0):
			self.bone.setPosition(pos)
		self.bone.save()

		if(model_ofs) {
			self.file.seek(model_ofs + self.pof, 0)
		}
		
		return None
	# }


# }
